using System;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;

namespace TestProject
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	class Driver
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			TestCase[] tests = {
				new PrimitiveBoolTest(),
				new PrimitiveByteTest(),
				new PrimitiveCharTest(),
				new PrimitiveDateTimeTest(),
				new PrimitiveDecimalTest(),
				new PrimitiveDoubleTest(),
				new PrimitiveInt16Test(),
				new PrimitiveInt32Test(),
				new PrimitiveInt64Test(),
				new PrimitiveSByteTest(),
				new PrimitiveSingleTest(),
				new PrimitiveTimeSpanTest(),
				new PrimitiveUInt16Test(),
				new PrimitiveUInt32Test(),
				new PrimitiveUInt64Test(),
				new SimpleStringTest(),
				new SimpleObjectTest(),
			    new EnumTest(),
				// Arrays
				new EmptyStringArrayTest(),
				new SimpleStringArrayTest(),
				new EmptyObjectArrayTest(),
				new SimpleObjectArrayTest(),
			    new EmptyEnumArrayTest(),
				new SimpleEnumArrayTest(),
				// mixed bag
				new MixedBagTest()
			};

			int oks = 0;

			foreach(TestCase test in tests) 
			{
				Console.Write(test.Name+"...");

				try 
				{
					if(test.Run()) 
					{
						oks++;
						Console.WriteLine("passed");
					} 
					else 
					{
						Console.WriteLine("failed");
					}
				} 
				catch(Exception e) 
				{
					Console.WriteLine("exception "+e.ToString());
				}
			}
			Console.WriteLine("\n>>> {0} out {1} passed!", oks, tests.Length);
			Console.ReadLine();
		}
	}

	abstract class TestCase 
	{
		public abstract Object GetData();
		public abstract String Name{get;}
		public abstract bool Check(Object original, Object copy);

		public bool Run() 
		{
			bool ret;

			Object original = GetData();
			BinaryFormatter formatter = new BinaryFormatter();
			MemoryStream ms = new MemoryStream();

			formatter.Serialize(ms, original);
			ms.Seek(0, SeekOrigin.Begin);
			Object copy = formatter.Deserialize(ms);

			ret = Check(original, copy);

			if(!ret) 
			{
				Console.WriteLine("\n>>> "+original);
				Console.WriteLine(">>> "+copy);
			}
			return ret;
		}
	}

	[Serializable]
	enum TestEnum 
	{
		first,
		second,
		third
	}
}
