using System;
using System.Runtime.CompilerServices;

namespace TestProject
{
	class MixedBagTest : TestCase 
	{
		public override Object GetData() 
		{
			return new Container();
		}

		public override bool Check(Object original, Object copy)
		{
			return original.ToString().Equals(copy.ToString());
		}

		public override String Name 
		{
			get 
			{
				return "mixed bag test";
			}
		}
	}

	/// <summary>
	/// tree-root, references other objects
	/// </summary>
	[Serializable]
	class Container 
	{
		public static int number = 0;
		public int ix = 23;
		PrimitiveTypes pt = new PrimitiveTypes();
		public String sx = "gibbet nix x";
		ArrayTypes at = new ArrayTypes();
		public int iy = 42;
		ObjectContainer oc = new ObjectContainer();
		public String sy = "gibbet nix y";
		public Object nullTest1 = null;
		TypedArray ta = new TypedArray();
		public int iz = 47;
		public Gna.InheritedClass ihd = new Gna.InheritedClass();
		public Foo.InheritingClass ihg = new Foo.InheritingClass();
		public Gna.InheritingClass shc = new Gna.InheritingClass();
		public Foo.InheritedClass fin = new Foo.InheritedClass();

		public override String ToString() 
		{
			return pt.ToString() + "\n"
				+at.ToString() + "\n"
				+oc.ToString() + "\n" 
				+ta.ToString() + "\n"
				+ihd.ToString() + "\n"
				+ihg.ToString() + "\n"
				+shc.ToString() + "\n"
				+fin.ToString() + "\n"
				+ix+"\n"
				+sx+"\n"
				+iy+"\n"
				+sy+"\n"
				+nullTest1+"\n"
				+iz+"\n";
		}
	}

	/// <summary>
	/// some primitive types
	/// </summary>
	[Serializable]
	class PrimitiveTypes 
	{
		public bool boolValue = true;
		public byte byteValue= 47;
		public char charValue= '2';
		public String nullTest2 = null;
		public decimal decimalValue = 48;
		public double doubleValue = 49;
		public Int16 int16Value = 50;
		public Int32 int32Value= 51;
		public Int64 int64Value= 52;
		public sbyte sbyteValue= 53;
		public Single singleValue= 54;
		public TimeSpan timespanValue= new TimeSpan(55);
		public DateTime datetimeValue= new DateTime(56);
		public UInt16 uint16Value = 57;
		public UInt32 uint32Value = 58;
		public UInt64 uint64Value= 59;
		public String stringValue= "string1";

		public override String ToString() 
		{
			return 
				boolValue+"\n"+
				byteValue+"\n"+
				charValue+"\n"+
				decimalValue+"\n"+
				doubleValue+"\n"+
				int16Value+"\n"+
				int32Value+"\n"+
				int64Value+"\n"+
				sbyteValue+"\n"+
				singleValue+"\n"+
				timespanValue+"\n"+
				datetimeValue+"\n"+
				uint16Value+"\n"+
				uint32Value+"\n"+
				uint64Value+"\n"+
				stringValue+"\n";
		}
	}

	/// <summary>
	/// some funny arrays
	/// </summary>
	[Serializable]
	class ArrayTypes 
	{
		public bool[] boolArray = {true, false, true, false};
		public int[] intArray = {1,2,3,4};
		public double[] nullTest3 = null;
		public String[] stringArray = {"string1", null, "string3", "string4"};
		public String[] stringNullArray = {"string1", "string2", "string3", "string4"};
		public String[] emptyStringArray = new String[50];

		public override String ToString() 
		{
			return
				boolArray.Length+"\n"+
				boolArray[0]+"\n"+
				boolArray[1]+"\n"+
				boolArray[2]+"\n"+
				boolArray[3]+"\n"+
				intArray.Length+"\n"+
				intArray[0]+"\n"+
				intArray[1]+"\n"+
				intArray[2]+"\n"+
				intArray[3]+"\n"+
				stringArray.Length+"\n"+
				stringArray[0]+"\n"+
				stringArray[1]+"\n"+
				stringArray[2]+"\n"+
				stringArray[3]+"\n"+
				stringNullArray.Length+"\n"+
				stringNullArray[0]+"\n"+
				stringNullArray[1]+"\n"+
				stringNullArray[2]+"\n"+
				stringNullArray[3]+"\n";
		}
	}

	[Serializable]
	class ObjectContainer 
	{
		public Object obj = new Object();
		public Object[] nullTest4 = null;
		public Object[] objectArray;
	
		public ObjectContainer() 
		{
			objectArray = new Object[5];
			objectArray[0] = new Object();
			objectArray[1] = "string2";
			objectArray[2] = obj;
			objectArray[3] = "string4";
			objectArray[4] = null;
		}


		public override String ToString() 
		{
			return obj+"\n"+
				objectArray.Length+"\n"+
				objectArray[0]+"\n"+
				objectArray[1]+"\n"+
				objectArray[2]+"\n"+
				objectArray[3]+"\n";
		}
	}

	[Serializable]
	class TypedArray 
	{
		public OtherClass oc = new OtherClass();
		public OtherClass[] oca = {
									  new OtherClass(),
									  new OtherClass(),
									  new OtherClass()
								  };

		public override String ToString() 
		{
			return oc.ToString() + "\n"+
				oca[0].ToString() + "\n"+
				oca[1].ToString() + "\n"+
				oca[2].ToString() + "\n";
		}
	}

	[Serializable]
	class OtherClass 
	{
		public int first;
		public long second;

		public OtherClass() 
		{
			first = Container.number++;
			second = Container.number++;
		}

		public override String ToString() 
		{
			return "f="+first+", second="+second;
		}
	}

	namespace Gna
	{
		[Serializable]
		class InheritedClass 
		{
			public String doubleMemberName = "parent";
			private String dmn2 = "base";

			public String getHiddenMember() 
			{
				return dmn2;
			}

			public override String ToString() 
			{
				return "parent="+doubleMemberName;
			}
		}

		[Serializable]
		class InheritingClass : Foo.InheritingClass
		{
			private String dmn2 = "2 Ch";

			public String getHiddenMember3() 
			{
				return dmn2;
			}

			public override String ToString() 
			{
				return 
					"\nbase 1=" + (((Gna.InheritedClass) this).getHiddenMember())+
					"\nbase 2=" + (((Foo.InheritingClass) this).getHiddenMember2())+
					"\nchild=" + (dmn2);
			}
		}
	}

	namespace Foo
	{
		[Serializable]
		class InheritedClass : Gna.InheritingClass
		{
			private String dmn2 = "3 Ch";

			public override String ToString() 
			{
				return 
					"\nbase 1=" + (((Gna.InheritedClass) this).getHiddenMember())+
					"\nbase 2=" + (((Foo.InheritingClass) this).getHiddenMember2())+
					"\nbase 3=" + (((Gna.InheritingClass) this).getHiddenMember3())+
					"\nchild=" + (dmn2);
			}
		}

		[Serializable]
		class InheritingClass : Gna.InheritedClass
		{
			public String doubledMemberName = "child";
			private String dmn2 = "1 Ch";

			public String getHiddenMember2() 
			{
				return dmn2;
			}

			public override String ToString() 
			{
				return "->child="+doubledMemberName+"\n"
					+"->"+base.doubleMemberName
					+"->"+((Gna.InheritedClass)this).doubleMemberName
					+"->"+dmn2
					+"->"+getHiddenMember();
			}
		}
	}
}
