using System;

namespace TestProject
{
	abstract class PrimitiveTestCase : TestCase 
	{
		public override bool Check(Object original, Object copy)
		{
			return original.Equals(copy);
		}
	}

	class EnumTest : PrimitiveTestCase 
	{
		public override Object GetData() 
		{
			return TestEnum.first;
		}

		public override String Name 
		{
			get 
			{
				return "toplevel test for enums";
			}
		}
	}

	class SimpleObjectTest : TestCase 
	{
		public override Object GetData() 
		{
			return new Object();
		}

		public override bool Check(Object original, Object copy)
		{
			return original.GetType().Equals(copy.GetType());
		}

		public override String Name 
		{
			get 
			{
				return "toplevel test for object";
			}
		}
	}

	class SimpleStringTest : PrimitiveTestCase
	{
		public override Object GetData() 
		{
			return "string";
		}

		public override String Name 
		{
			get 
			{
				return "toplevel test for String";
			}
		}
	}

	class PrimitiveUInt64Test : PrimitiveTestCase 
	{
		public override Object GetData() 
		{
			return (ulong) 64;
		}

		public override String Name 
		{
			get 
			{
				return "toplevel primitive test for uint64";
			}
		}
	}

	class PrimitiveUInt32Test : PrimitiveTestCase 
	{
		public override Object GetData() 
		{
			return (uint) 32;
		}

		public override String Name 
		{
			get 
			{
				return "toplevel primitive test for uint32";
			}
		}
	}

	class PrimitiveUInt16Test : PrimitiveTestCase 
	{
		public override Object GetData() 
		{
			return (ushort) 16;
		}

		public override String Name 
		{
			get 
			{
				return "toplevel primitive test for uint16";
			}
		}
	}

	class PrimitiveTimeSpanTest : PrimitiveTestCase 
	{
		public override Object GetData() 
		{
			return new TimeSpan(1,2,3);
		}

		public override String Name 
		{
			get 
			{
				return "toplevel primitive test for timespan";
			}
		}
	}

	class PrimitiveSingleTest : PrimitiveTestCase 
	{
		public override Object GetData() 
		{
			return (float) 12.13;
		}

		public override String Name 
		{
			get 
			{
				return "toplevel primitive test for single";
			}
		}
	}

	class PrimitiveSByteTest : PrimitiveTestCase 
	{
		public override Object GetData() 
		{
			return (sbyte) 12;
		}

		public override String Name 
		{
			get 
			{
				return "toplevel primitive test for sbyte";
			}
		}
	}

	class PrimitiveInt64Test : PrimitiveTestCase 
	{
		public override Object GetData() 
		{
			return (long) 64;
		}

		public override String Name 
		{
			get 
			{
				return "toplevel primitive test for int64";
			}
		}
	}

	class PrimitiveInt32Test : PrimitiveTestCase 
	{
		public override Object GetData() 
		{
			return (int) 32;
		}

		public override String Name 
		{
			get 
			{
				return "toplevel primitive test for int32";
			}
		}
	}

	class PrimitiveInt16Test : PrimitiveTestCase 
	{
		public override Object GetData() 
		{
			return (short) 16;
		}

		public override String Name 
		{
			get 
			{
				return "toplevel primitive test for int16";
			}
		}
	}

	class PrimitiveDoubleTest : PrimitiveTestCase 
	{
		public override Object GetData() 
		{
			return (double) 47.11;
		}

		public override String Name 
		{
			get 
			{
				return "toplevel primitive test for double";
			}
		}
	}

	class PrimitiveDecimalTest : PrimitiveTestCase 
	{
		public override Object GetData() 
		{
			return (decimal) 47.11;
		}

		public override String Name 
		{
			get 
			{
				return "toplevel primitive test for decimal";
			}
		}
	}

	class PrimitiveDateTimeTest : PrimitiveTestCase 
	{
		public override Object GetData() 
		{
			return new DateTime(1999, 2, 23);
		}

		public override String Name 
		{
			get 
			{
				return "toplevel primitive test for datetime";
			}
		}
	}

	class PrimitiveCharTest : PrimitiveTestCase 
	{
		public override Object GetData() 
		{
			return 'a';
		}

		public override String Name 
		{
			get 
			{
				return "toplevel primitive test for char";
			}
		}
	}

	class PrimitiveByteTest : PrimitiveTestCase 
	{
		public override Object GetData() 
		{
			return (byte) 12;
		}

		public override String Name 
		{
			get 
			{
				return "toplevel primitive test for byte";
			}
		}
	}

	class PrimitiveBoolTest : PrimitiveTestCase 
	{
		public override Object GetData() 
		{
			return true;
		}

		public override String Name 
		{
			get 
			{
				return "toplevel primitive test for bool";
			}
		}
	}
}
